import time
from PyQt5.QtCore import QVariant
from qgis.core import QgsApplication,QgsCoordinateReferenceSystem,QgsCoordinateTransform,QgsFeature,QgsField,QgsGeometry,QgsMapLayer,QgsPointXY,QgsProject,QgsTask,QgsVectorLayer
from qgis.gui import QgisInterface,QgsMapCanvas
from ..com.orbitgis.tool.message.Message import Message
from ..orbitgt.mapping3d.sdk.viewer.AList import AList
from ..orbitgt.mapping3d.sdk.viewer.Bounds import Bounds
from ..orbitgt.mapping3d.sdk.viewer.MapObject import MapObject
from ..orbitgt.mapping3d.sdk.viewer.Viewer3DM import Viewer3DM
from .MapObjectLayer import MapObjectLayer
from .MapObjectRenderer import MapObjectRenderer
from .MapObjects import MapObjects

# Class UpdateGlobal manages the updates to the global layer
#
class UpdateGlobal:

    # Create a new updater
    #
    def __init__(self,viewer: Viewer3DM,layer: MapObjectLayer,iface: QgisInterface):
        self.viewer = viewer
        self.layer = layer
        self.iface = iface
        self.updateTime = 0.0
        self.lastRequestTime = 0.0
        self.isUpdating = False
        self.mapObjects = None

    # Called when the globals have been changed and the display need to be updated
    #
    def updatedGlobals(self):
        self.updateTime = time.time()

    # Check for updates
    #
    def checkForUpdates(self) -> None:
        # already updating?
        if (self.isUpdating):
            return
        # already up-to-date?
        if (self.lastRequestTime>self.updateTime):
            return
        currentTime = time.time()
        # throttle the rate of update requests
        if (self.lastRequestTime>0.0) and (currentTime<self.lastRequestTime+0.1):
            return
        # throttle the rate of measure changes
        if (self.updateTime>0.0) and (currentTime<self.updateTime+0.1):
            return
        # start the update task (to free the calling timer thread)
        self.isUpdating = True
        self.lastRequestTime = currentTime
        self.updateTask = QgsTask.fromFunction("Update globals",self.update1,on_finished=self.update2)
        QgsApplication.taskManager().addTask(self.updateTask)

    # Update (in background thread)
    #
    def update1(self,task: QgsTask):
        try:
            Message.print(__name__,"Requesting globals")
            self.mapObjects = self.viewer.getGlobals()
            if (self.mapObjects!=None): Message.print(__name__,"Got "+str(self.mapObjects.size())+" mapobjects")
            self.isUpdating = False
            return
        except Exception as e:
            Message.printError(__name__,e,"Error while updating globals")
            self.isUpdating = False
            return
    
    # Finish the update (called by the main UI thread)
    #
    def update2(self,taskException):
        self.layer.update(self.mapObjects,True)
